import logging

def verbosity_to_loglevel(verbosity=0):
    verbosity = verbosity or 0
    levels = [
        logging.WARNING,
        logging.INFO,
        logging.DEBUG,
    ]
    if verbosity >= len(levels):
        verbosity = -1
    level = levels[verbosity]
    return level

def set_loglevel(loglevel=logging.WARNING, logger=''):
    return logging.getLogger(logger).setLevel(loglevel)

def set_verbosity(verbosity=0):
    loglevel = verbosity_to_loglevel(verbosity)
    set_loglevel(loglevel)
    return loglevel



from concurrent import futures

import atexit
import grpc

from koapy.grpc import KiwoomOpenApiService_pb2_grpc
from koapy.grpc.KiwoomOpenApiServiceServicer import KiwoomOpenApiServiceServicer

from koapy.config import config
from koapy.utils.networking import get_free_localhost_port

class KiwoomOpenApiServiceServer:

    def __init__(self, control, host=None, port=None, max_workers=None):
        self._control = control
        if host is None:
            host = config.get_string('koapy.grpc.host', 'localhost')
        if port is None:
            port = config.get('koapy.grpc.port')
        if port == 0:
            port = get_free_localhost_port()

        self._host = host
        self._port = port

        if self._port is None:
            raise ValueError('Argument port cannot be None')

        if max_workers is None:
            max_workers = config.get_int('koapy.grpc.server.max_workers', 8)

        self._max_workers = max_workers

        self._servicer = KiwoomOpenApiServiceServicer(control)
        self._executor = futures.ThreadPoolExecutor(max_workers=self._max_workers)
        atexit.register(self._executor.shutdown, False)
        self._server = grpc.server(self._executor)
        KiwoomOpenApiService_pb2_grpc.add_KiwoomOpenApiServiceServicer_to_server(self._servicer, self._server)

        self._target = self._host + ':' + str(self._port)
        self._server.add_insecure_port(self._target)

    def get_host(self):
        return self._host

    def get_port(self):
        return self._port

    def start(self):
        return self._server.start()

    def wait_for_termination(self, timeout=None):
        return self._server.wait_for_termination(timeout)

    def stop(self, grace=None):
        rcode = self._server.stop(grace)
        return rcode

    def __getattr__(self, name):
        return getattr(self._server, name)



import os
import subprocess

import sys

if sys.platform == 'win32':
    from mslex import quote
else:
    from shlex import quote

def compile_proto():
    proto_filename = 'KiwoomOpenApiService.proto'
    file_dir = os.path.dirname(os.path.realpath(__file__))
    project_dir = os.path.realpath(os.path.join(file_dir, '..', '..', '..'))
    proto_path = project_dir
    proto_filepath = os.path.join(proto_path, 'koapy', 'grpc', proto_filename)
    python_out = proto_path
    grpc_python_out = proto_path
    cmd = [
        'python',
        '-m', 'grpc_tools.protoc',
        '--proto_path=%s' % quote(proto_path),
        '--python_out=%s' % quote(python_out),
        '--grpc_python_out=%s' % quote(grpc_python_out),
        proto_filepath]
    print(' '.join(cmd))
    subprocess.run(cmd, cwd=project_dir, check=True)

if __name__ == '__main__':
    compile_proto()



from koapy.grpc import KiwoomOpenApiService_pb2

class KiwoomOpenApiServiceClientSideDynamicCallable:

    def __init__(self, stub, name):
        self._stub = stub
        self._name = name

    @classmethod
    def _createCallRequest(cls, name, args):
        request = KiwoomOpenApiService_pb2.CallRequest()
        request.name = name
        for i, arg in enumerate(args):
            if isinstance(arg, str):
                request.arguments.add().string_value = arg # pylint: disable=no-member
            elif isinstance(arg, int):
                request.arguments.add().long_value = arg # pylint: disable=no-member
            else:
                raise TypeError('Unexpected type for argument %d: %s' % (i, type(arg)))
        return request

    @classmethod
    def _unpackResponse(cls, response):
        if response.return_value.HasField('string_value'):
            return response.return_value.string_value
        elif response.return_value.HasField('long_value'):
            return response.return_value.long_value
        else:
            return None

    def __call__(self, *args):
        request = self._createCallRequest(self._name, args)
        response = self._stub.Call(request)
        return self._unpackResponse(response)



from abc import ABC
from contextlib import ExitStack
from threading import RLock

from koapy.grpc.event.KiwoomOpenApiEventHandlerFunctions import KiwoomOpenApiEventHandlerFunctions
from koapy.grpc.observer.QueueBasedIterableObserver import QueueBasedIterableObserver

from koapy.utils.notimplemented import isimplemented

class BaseKiwoomOpenApiEventHandler(KiwoomOpenApiEventHandlerFunctions, ABC):

    def __init__(self, control):
        self._control = control
        self._observer = QueueBasedIterableObserver()
        self._enter_count = 0
        self._stack = ExitStack()
        self._lock = RLock()

    @property
    def control(self):
        return self._control

    @property
    def observer(self):
        return self._observer

    def on_next(self, value):
        self.observer.on_next(value)

    def on_error(self, error):
        self.observer.on_error(error)

    def on_completed(self):
        self.observer.on_completed()

    def __getattr__(self, name):
        return getattr(self.control, name)

    @classmethod
    def names(cls):
        names = [name for name in dir(KiwoomOpenApiEventHandlerFunctions) if name.startswith('On')]
        return names

    def slots(self):
        names = self.names()
        slots = [getattr(self, name) for name in names]
        names_and_slots_implemented = [(name, slot) for name, slot in zip(names, slots) if isimplemented(slot)]
        return names_and_slots_implemented

    def connect(self):
        for name, slot in self.slots():
            getattr(self.control, name).connect(slot)

    def disconnect(self):
        for name, slot in self.slots():
            getattr(self.control, name).disconnect(slot)

    def on_enter(self):
        pass

    def on_exit(self):
        pass

    def add_callback(self, callback, *args, **kwargs):
        self._stack.callback(callback, *args, **kwargs)

    def __enter__(self):
        with self._lock:
            if self._enter_count == 0:
                self.connect()
                self.on_enter()
            self._enter_count += 1
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        with self._lock:
            if self._enter_count > 0:
                self._enter_count -= 1
            if self._enter_count == 0:
                self.disconnect()
                self.on_exit()
                self._stack.__exit__(exc_type, exc_value, traceback)

    def __iter__(self):
        with self:
            return iter(self.observer)



import os
import json
import contextlib
import pandas as pd

from koapy.utils.serialization import JsonSerializable

class RealType(JsonSerializable):

    class Fid(JsonSerializable):

        __outer_class__ = None

        _FID_DUMP_FILENAME = 'fid.xlsx'
        _NAME_BY_FID = {}

        def __init__(self, fid=None, name=None):
            self.fid = int(fid)
            self.name = name

        def __repr__(self):
            return '%s.%s(%r, %r)' % (
                self.__outer_class__.__name__,
                self.__class__.__name__,
                self.fid,
                self.name)

        @classmethod
        def name_by_fid_from_dump_file(cls, dump_file=None):
            if dump_file is None:
                dump_file = os.path.join(os.path.dirname(__file__), 'data', cls._FID_DUMP_FILENAME)
            df = pd.read_excel(dump_file)
            fids = [cls(pair[0], pair[1]) for pair in zip(df['fid'], df['name'])]
            result = {fid.fid: fid.name for fid in fids}
            return result

        @classmethod
        def load_from_dump_file(cls, dump_file=None):
            cls._NAME_BY_FID = cls.name_by_fid_from_dump_file(dump_file)

        @classmethod
        def get_name_by_fid(cls, fid, default=None):
            fid = int(fid)
            return cls._NAME_BY_FID.get(fid, default)

    _REALTYPE_BY_DESC_DUMP_FILENAME = 'realtype_by_desc.json'
    _REALTYPE_BY_DESC = {}

    def __init__(self, gidc=None, desc=None, nfid=None, fids=None):
        self.gidc = gidc
        self.desc = desc
        self.nfid = nfid
        self.fids = fids

    def __repr__(self):
        return '%s(%r, %r, %r, %r)' % (
            self.__class__.__name__,
            self.gidc,
            self.desc,
            self.nfid,
            self.fids)

    @classmethod
    def get_fids_by_realtype(cls, realtype):
        result = cls._REALTYPE_BY_DESC.get(realtype)
        if result is not None:
            return result.fids
        return None

    @classmethod
    def get_fids_by_realtype_as_string(cls, realtype):
        fids = cls.get_fids_by_realtype(realtype)
        if fids is not None:
            fids = [str(fid) for fid in fids]
            fids = ';'.join(fids)
        return fids

    @classmethod
    def get_field_names_by_realtype(cls, realtype):
        fids = cls.get_fids_by_realtype(realtype)
        if fids is not None:
            names = [cls.Fid.get_name_by_fid(fid, str(fid)) for fid in fids]
            return names
        return None

    @classmethod
    def realtypes_from_datfile(cls, f=None, encoding='euc-kr'):
        gidc_width = 2
        desc_width = 20
        nfid_width = 3
        fid_width = 5
        if f is None:
            f = r'C:\OpenAPI\data\nkrealtime.dat'
        with contextlib.ExitStack() as stack:
            if isinstance(f, str):
                f = stack.enter_context(open(f, 'rb'))
            lines = iter(f)
            lines = map(lambda line: line.rstrip(b'\r\n'), lines)
            lines = filter(lambda line: not line.startswith(b';'), lines)
            realtypes = []
            for line in lines:
                gidc = line[:gidc_width]
                line = line[gidc_width:]
                gidc = gidc.decode(encoding).strip()
                desc = line[:desc_width]
                line = line[desc_width:]
                desc = desc.decode(encoding).strip()
                nfid = line[:nfid_width]
                nfid = nfid.strip()
                line = line[nfid_width:]
                if len(nfid) > 0:
                    nfid = int(nfid)
                else:
                    nfid = 0
                fids = []
                for _ in range(nfid):
                    fid = line[:fid_width]
                    line = line[fid_width:]
                    fid = int(fid)
                    fids.append(fid)
                assert len(line.rstrip(b'^M ')) == 0
                realtypes.append(cls(gidc, desc, nfid, fids))
            return realtypes

    @classmethod
    def realtype_by_desc_from_datfile(cls, f=None):
        realtypes = cls.realtypes_from_datfile(f)
        result = {realtype.desc:realtype for realtype in realtypes}
        return result

    @classmethod
    def dump_realtype_by_desc(cls, dump_file=None):
        with contextlib.ExitStack() as stack:
            if dump_file is None:
                dump_file = os.path.join(os.path.dirname(__file__), 'data', cls._REALTYPE_BY_DESC_DUMP_FILENAME)
            if isinstance(dump_file, str):
                dump_file = stack.enter_context(open(dump_file, 'w'))
            result = cls.realtype_by_desc_from_datfile()
            for tr_code in result:
                result[tr_code] = result[tr_code].to_dict()
            return json.dump(result, dump_file)

    @classmethod
    def realtype_by_desc_from_dump_file(cls, dump_file=None):
        with contextlib.ExitStack() as stack:
            if dump_file is None:
                dump_file = os.path.join(os.path.dirname(__file__), 'data', cls._REALTYPE_BY_DESC_DUMP_FILENAME)
            if isinstance(dump_file, str) and os.path.exists(dump_file) and os.stat(dump_file).st_size > 0:
                dump_file = stack.enter_context(open(dump_file, 'r'))
                result = json.load(dump_file)
                for desc in result:
                    result[desc] = cls.from_dict(result[desc])
                return result
        return {}

    @classmethod
    def load_from_dump_file(cls, dump_file=None):
        cls._REALTYPE_BY_DESC = cls.realtype_by_desc_from_dump_file(dump_file)
        cls.Fid.load_from_dump_file()

RealType.Fid.__outer_class__ = RealType

RealType.load_from_dump_file()

def main():
    RealType.dump_realtype_by_desc()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: koapy/grpc/KiwoomOpenApiService.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='koapy/grpc/KiwoomOpenApiService.proto',
  package='koapy.grpc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n%koapy/grpc/KiwoomOpenApiService.proto\x12\nkoapy.grpc\"A\n\x08\x41rgument\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x14\n\nlong_value\x18\x02 \x01(\x03H\x00\x42\x07\n\x05value\"D\n\x0b\x43\x61llRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\'\n\targuments\x18\x02 \x03(\x0b\x32\x14.koapy.grpc.Argument\"D\n\x0bReturnValue\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x14\n\nlong_value\x18\x02 \x01(\x03H\x00\x42\x07\n\x05value\"=\n\x0c\x43\x61llResponse\x12-\n\x0creturn_value\x18\x01 \x01(\x0b\x32\x17.koapy.grpc.ReturnValue\"*\n\rListenRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05slots\x18\x02 \x03(\t\"+\n\nSingleData\x12\r\n\x05names\x18\x01 \x03(\t\x12\x0e\n\x06values\x18\x02 \x03(\t\" \n\x0eRepeatedString\x12\x0e\n\x06values\x18\x01 \x03(\t\"F\n\tMultiData\x12\r\n\x05names\x18\x01 \x03(\t\x12*\n\x06values\x18\x02 \x03(\x0b\x32\x1a.koapy.grpc.RepeatedString\"G\n\x0eListenResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\'\n\targuments\x18\x02 \x03(\x0b\x32\x14.koapy.grpc.Argument\"\x1f\n\x11StopListenRequest\x12\n\n\x02id\x18\x01 \x01(\t\"(\n\x12StopListenResponse\x12\x12\n\nsuccessful\x18\x01 \x01(\x08\"7\n\x13\x43ustomListenRequest\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x12\n\nclass_name\x18\x02 \x01(\t\"\xa5\x01\n\x14\x43ustomListenResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\'\n\targuments\x18\x02 \x03(\x0b\x32\x14.koapy.grpc.Argument\x12+\n\x0bsingle_data\x18\x03 \x01(\x0b\x32\x16.koapy.grpc.SingleData\x12)\n\nmulti_data\x18\x04 \x01(\x0b\x32\x15.koapy.grpc.MultiData\"\x84\x01\n\x1a\x43ustomCallAndListenRequest\x12-\n\x0c\x63\x61ll_request\x18\x01 \x01(\x0b\x32\x17.koapy.grpc.CallRequest\x12\x37\n\x0elisten_request\x18\x02 \x01(\x0b\x32\x1f.koapy.grpc.CustomListenRequest\"\x99\x01\n\x1b\x43ustomCallAndListenResponse\x12\x31\n\rcall_response\x18\x01 \x01(\x0b\x32\x18.koapy.grpc.CallResponseH\x00\x12;\n\x0flisten_response\x18\x02 \x01(\x0b\x32 .koapy.grpc.CustomListenResponseH\x00\x42\n\n\x08response\"\x0e\n\x0cLoginRequest\"S\n\x1dTransactionSubscriptionTarget\x12\x0b\n\x03RET\x18\x01 \x01(\x08\x12\n\n\x02TR\x18\x02 \x01(\x08\x12\x0c\n\x04REAL\x18\x03 \x01(\x08\x12\x0b\n\x03MSG\x18\x04 \x01(\x08\"z\n\x18TransactionStopCondition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x41\n\ncomparator\x18\x03 \x01(\x0e\x32-.koapy.grpc.TransactionStopConditionCompartor\"\xbb\x02\n\x12TransactionRequest\x12\x14\n\x0crequest_name\x18\x01 \x01(\t\x12\x18\n\x10transaction_code\x18\x02 \x01(\t\x12\x11\n\tscreen_no\x18\x03 \x01(\t\x12:\n\x06inputs\x18\x04 \x03(\x0b\x32*.koapy.grpc.TransactionRequest.InputsEntry\x12\x39\n\x06output\x18\x05 \x01(\x0b\x32).koapy.grpc.TransactionSubscriptionTarget\x12<\n\x0estop_condition\x18\x06 \x01(\x0b\x32$.koapy.grpc.TransactionStopCondition\x1a-\n\x0bInputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"]\n\x17OrderSubscriptionTarget\x12\x0b\n\x03RET\x18\x01 \x01(\x08\x12\n\n\x02TR\x18\x02 \x01(\x08\x12\x0c\n\x04REAL\x18\x03 \x01(\x08\x12\x0b\n\x03MSG\x18\x04 \x01(\x08\x12\x0e\n\x06\x43HEJAN\x18\x05 \x01(\x08\"\xf2\x01\n\x0cOrderRequest\x12\x14\n\x0crequest_name\x18\x01 \x01(\t\x12\x11\n\tscreen_no\x18\x02 \x01(\t\x12\x12\n\naccount_no\x18\x03 \x01(\t\x12\x12\n\norder_type\x18\x04 \x01(\x03\x12\x0c\n\x04\x63ode\x18\x05 \x01(\t\x12\x10\n\x08quantity\x18\x06 \x01(\x03\x12\r\n\x05price\x18\x07 \x01(\x03\x12\x12\n\nquote_type\x18\x08 \x01(\t\x12\x19\n\x11original_order_no\x18\t \x01(\t\x12\x33\n\x06target\x18\n \x01(\x0b\x32#.koapy.grpc.OrderSubscriptionTarget\"R\n\x10RealRequestFlags\x12\x12\n\ninfer_fids\x18\x01 \x01(\x08\x12\x16\n\x0ereadable_names\x18\x02 \x01(\x08\x12\x12\n\nfast_parse\x18\x03 \x01(\x08\"\x85\x01\n\x0bRealRequest\x12\x11\n\tscreen_no\x18\x01 \x01(\t\x12\x11\n\tcode_list\x18\x02 \x03(\t\x12\x10\n\x08\x66id_list\x18\x03 \x03(\x05\x12\x11\n\treal_type\x18\x04 \x01(\t\x12+\n\x05\x66lags\x18\x05 \x01(\x0b\x32\x1c.koapy.grpc.RealRequestFlags\"3\n\x12SetLogLevelRequest\x12\r\n\x05level\x18\x01 \x01(\x05\x12\x0e\n\x06logger\x18\x02 \x01(\t\"\x15\n\x13SetLogLevelResponse*\x9d\x01\n!TransactionStopConditionCompartor\x12\x19\n\x15LESS_THAN_OR_EQUAL_TO\x10\x00\x12\r\n\tLESS_THAN\x10\x01\x12\x1c\n\x18GREATER_THAN_OR_EQUAL_TO\x10\x02\x12\x10\n\x0cGREATER_THAN\x10\x03\x12\x0c\n\x08\x45QUAL_TO\x10\x04\x12\x10\n\x0cNOT_EQUAL_TO\x10\x05\x32\xdc\x06\n\x14KiwoomOpenApiService\x12;\n\x04\x43\x61ll\x12\x17.koapy.grpc.CallRequest\x1a\x18.koapy.grpc.CallResponse\"\x00\x12I\n\x06Listen\x12\x19.koapy.grpc.ListenRequest\x1a .koapy.grpc.CustomListenResponse\"\x00\x30\x01\x12M\n\nStopListen\x12\x1d.koapy.grpc.StopListenRequest\x1a\x1e.koapy.grpc.StopListenResponse\"\x00\x12U\n\x0c\x43ustomListen\x12\x1f.koapy.grpc.CustomListenRequest\x1a .koapy.grpc.CustomListenResponse\"\x00\x30\x01\x12j\n\x13\x43ustomCallAndListen\x12&.koapy.grpc.CustomCallAndListenRequest\x1a\'.koapy.grpc.CustomCallAndListenResponse\"\x00\x30\x01\x12R\n\tLoginCall\x12\x18.koapy.grpc.LoginRequest\x1a\'.koapy.grpc.CustomCallAndListenResponse\"\x00\x30\x01\x12^\n\x0fTransactionCall\x12\x1e.koapy.grpc.TransactionRequest\x1a\'.koapy.grpc.CustomCallAndListenResponse\"\x00\x30\x01\x12R\n\tOrderCall\x12\x18.koapy.grpc.OrderRequest\x1a\'.koapy.grpc.CustomCallAndListenResponse\"\x00\x30\x01\x12P\n\x08RealCall\x12\x17.koapy.grpc.RealRequest\x1a\'.koapy.grpc.CustomCallAndListenResponse\"\x00\x30\x01\x12P\n\x0bSetLogLevel\x12\x1e.koapy.grpc.SetLogLevelRequest\x1a\x1f.koapy.grpc.SetLogLevelResponse\"\x00\x62\x06proto3'
)

_TRANSACTIONSTOPCONDITIONCOMPARTOR = _descriptor.EnumDescriptor(
  name='TransactionStopConditionCompartor',
  full_name='koapy.grpc.TransactionStopConditionCompartor',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LESS_THAN_OR_EQUAL_TO', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LESS_THAN', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_THAN_OR_EQUAL_TO', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREATER_THAN', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQUAL_TO', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_EQUAL_TO', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2362,
  serialized_end=2519,
)
_sym_db.RegisterEnumDescriptor(_TRANSACTIONSTOPCONDITIONCOMPARTOR)

TransactionStopConditionCompartor = enum_type_wrapper.EnumTypeWrapper(_TRANSACTIONSTOPCONDITIONCOMPARTOR)
LESS_THAN_OR_EQUAL_TO = 0
LESS_THAN = 1
GREATER_THAN_OR_EQUAL_TO = 2
GREATER_THAN = 3
EQUAL_TO = 4
NOT_EQUAL_TO = 5



_ARGUMENT = _descriptor.Descriptor(
  name='Argument',
  full_name='koapy.grpc.Argument',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_value', full_name='koapy.grpc.Argument.string_value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='long_value', full_name='koapy.grpc.Argument.long_value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='koapy.grpc.Argument.value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=53,
  serialized_end=118,
)


_CALLREQUEST = _descriptor.Descriptor(
  name='CallRequest',
  full_name='koapy.grpc.CallRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='koapy.grpc.CallRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arguments', full_name='koapy.grpc.CallRequest.arguments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=188,
)


_RETURNVALUE = _descriptor.Descriptor(
  name='ReturnValue',
  full_name='koapy.grpc.ReturnValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_value', full_name='koapy.grpc.ReturnValue.string_value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='long_value', full_name='koapy.grpc.ReturnValue.long_value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='koapy.grpc.ReturnValue.value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=190,
  serialized_end=258,
)


_CALLRESPONSE = _descriptor.Descriptor(
  name='CallResponse',
  full_name='koapy.grpc.CallResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_value', full_name='koapy.grpc.CallResponse.return_value', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=260,
  serialized_end=321,
)


_LISTENREQUEST = _descriptor.Descriptor(
  name='ListenRequest',
  full_name='koapy.grpc.ListenRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='koapy.grpc.ListenRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slots', full_name='koapy.grpc.ListenRequest.slots', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=323,
  serialized_end=365,
)


_SINGLEDATA = _descriptor.Descriptor(
  name='SingleData',
  full_name='koapy.grpc.SingleData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='names', full_name='koapy.grpc.SingleData.names', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='values', full_name='koapy.grpc.SingleData.values', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=367,
  serialized_end=410,
)


_REPEATEDSTRING = _descriptor.Descriptor(
  name='RepeatedString',
  full_name='koapy.grpc.RepeatedString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='koapy.grpc.RepeatedString.values', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=412,
  serialized_end=444,
)


_MULTIDATA = _descriptor.Descriptor(
  name='MultiData',
  full_name='koapy.grpc.MultiData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='names', full_name='koapy.grpc.MultiData.names', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='values', full_name='koapy.grpc.MultiData.values', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=446,
  serialized_end=516,
)


_LISTENRESPONSE = _descriptor.Descriptor(
  name='ListenResponse',
  full_name='koapy.grpc.ListenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='koapy.grpc.ListenResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arguments', full_name='koapy.grpc.ListenResponse.arguments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=518,
  serialized_end=589,
)


_STOPLISTENREQUEST = _descriptor.Descriptor(
  name='StopListenRequest',
  full_name='koapy.grpc.StopListenRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='koapy.grpc.StopListenRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=591,
  serialized_end=622,
)


_STOPLISTENRESPONSE = _descriptor.Descriptor(
  name='StopListenResponse',
  full_name='koapy.grpc.StopListenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='successful', full_name='koapy.grpc.StopListenResponse.successful', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=624,
  serialized_end=664,
)


_CUSTOMLISTENREQUEST = _descriptor.Descriptor(
  name='CustomListenRequest',
  full_name='koapy.grpc.CustomListenRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='koapy.grpc.CustomListenRequest.code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='class_name', full_name='koapy.grpc.CustomListenRequest.class_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=666,
  serialized_end=721,
)


_CUSTOMLISTENRESPONSE = _descriptor.Descriptor(
  name='CustomListenResponse',
  full_name='koapy.grpc.CustomListenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='koapy.grpc.CustomListenResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arguments', full_name='koapy.grpc.CustomListenResponse.arguments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='single_data', full_name='koapy.grpc.CustomListenResponse.single_data', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multi_data', full_name='koapy.grpc.CustomListenResponse.multi_data', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=724,
  serialized_end=889,
)


_CUSTOMCALLANDLISTENREQUEST = _descriptor.Descriptor(
  name='CustomCallAndListenRequest',
  full_name='koapy.grpc.CustomCallAndListenRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='call_request', full_name='koapy.grpc.CustomCallAndListenRequest.call_request', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='listen_request', full_name='koapy.grpc.CustomCallAndListenRequest.listen_request', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=892,
  serialized_end=1024,
)


_CUSTOMCALLANDLISTENRESPONSE = _descriptor.Descriptor(
  name='CustomCallAndListenResponse',
  full_name='koapy.grpc.CustomCallAndListenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='call_response', full_name='koapy.grpc.CustomCallAndListenResponse.call_response', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='listen_response', full_name='koapy.grpc.CustomCallAndListenResponse.listen_response', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='response', full_name='koapy.grpc.CustomCallAndListenResponse.response',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1027,
  serialized_end=1180,
)


_LOGINREQUEST = _descriptor.Descriptor(
  name='LoginRequest',
  full_name='koapy.grpc.LoginRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1182,
  serialized_end=1196,
)


_TRANSACTIONSUBSCRIPTIONTARGET = _descriptor.Descriptor(
  name='TransactionSubscriptionTarget',
  full_name='koapy.grpc.TransactionSubscriptionTarget',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='RET', full_name='koapy.grpc.TransactionSubscriptionTarget.RET', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TR', full_name='koapy.grpc.TransactionSubscriptionTarget.TR', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='REAL', full_name='koapy.grpc.TransactionSubscriptionTarget.REAL', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='MSG', full_name='koapy.grpc.TransactionSubscriptionTarget.MSG', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1198,
  serialized_end=1281,
)


_TRANSACTIONSTOPCONDITION = _descriptor.Descriptor(
  name='TransactionStopCondition',
  full_name='koapy.grpc.TransactionStopCondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='koapy.grpc.TransactionStopCondition.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='koapy.grpc.TransactionStopCondition.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='comparator', full_name='koapy.grpc.TransactionStopCondition.comparator', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1283,
  serialized_end=1405,
)


_TRANSACTIONREQUEST_INPUTSENTRY = _descriptor.Descriptor(
  name='InputsEntry',
  full_name='koapy.grpc.TransactionRequest.InputsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='koapy.grpc.TransactionRequest.InputsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='koapy.grpc.TransactionRequest.InputsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1678,
  serialized_end=1723,
)

_TRANSACTIONREQUEST = _descriptor.Descriptor(
  name='TransactionRequest',
  full_name='koapy.grpc.TransactionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_name', full_name='koapy.grpc.TransactionRequest.request_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transaction_code', full_name='koapy.grpc.TransactionRequest.transaction_code', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='screen_no', full_name='koapy.grpc.TransactionRequest.screen_no', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inputs', full_name='koapy.grpc.TransactionRequest.inputs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output', full_name='koapy.grpc.TransactionRequest.output', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stop_condition', full_name='koapy.grpc.TransactionRequest.stop_condition', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_TRANSACTIONREQUEST_INPUTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1408,
  serialized_end=1723,
)


_ORDERSUBSCRIPTIONTARGET = _descriptor.Descriptor(
  name='OrderSubscriptionTarget',
  full_name='koapy.grpc.OrderSubscriptionTarget',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='RET', full_name='koapy.grpc.OrderSubscriptionTarget.RET', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TR', full_name='koapy.grpc.OrderSubscriptionTarget.TR', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='REAL', full_name='koapy.grpc.OrderSubscriptionTarget.REAL', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='MSG', full_name='koapy.grpc.OrderSubscriptionTarget.MSG', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='CHEJAN', full_name='koapy.grpc.OrderSubscriptionTarget.CHEJAN', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1725,
  serialized_end=1818,
)


_ORDERREQUEST = _descriptor.Descriptor(
  name='OrderRequest',
  full_name='koapy.grpc.OrderRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_name', full_name='koapy.grpc.OrderRequest.request_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='screen_no', full_name='koapy.grpc.OrderRequest.screen_no', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='account_no', full_name='koapy.grpc.OrderRequest.account_no', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_type', full_name='koapy.grpc.OrderRequest.order_type', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='koapy.grpc.OrderRequest.code', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quantity', full_name='koapy.grpc.OrderRequest.quantity', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='koapy.grpc.OrderRequest.price', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quote_type', full_name='koapy.grpc.OrderRequest.quote_type', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='original_order_no', full_name='koapy.grpc.OrderRequest.original_order_no', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target', full_name='koapy.grpc.OrderRequest.target', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1821,
  serialized_end=2063,
)


_REALREQUESTFLAGS = _descriptor.Descriptor(
  name='RealRequestFlags',
  full_name='koapy.grpc.RealRequestFlags',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='infer_fids', full_name='koapy.grpc.RealRequestFlags.infer_fids', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='readable_names', full_name='koapy.grpc.RealRequestFlags.readable_names', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fast_parse', full_name='koapy.grpc.RealRequestFlags.fast_parse', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2065,
  serialized_end=2147,
)


_REALREQUEST = _descriptor.Descriptor(
  name='RealRequest',
  full_name='koapy.grpc.RealRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='screen_no', full_name='koapy.grpc.RealRequest.screen_no', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code_list', full_name='koapy.grpc.RealRequest.code_list', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fid_list', full_name='koapy.grpc.RealRequest.fid_list', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='real_type', full_name='koapy.grpc.RealRequest.real_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flags', full_name='koapy.grpc.RealRequest.flags', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2150,
  serialized_end=2283,
)


_SETLOGLEVELREQUEST = _descriptor.Descriptor(
  name='SetLogLevelRequest',
  full_name='koapy.grpc.SetLogLevelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='level', full_name='koapy.grpc.SetLogLevelRequest.level', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='logger', full_name='koapy.grpc.SetLogLevelRequest.logger', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2285,
  serialized_end=2336,
)


_SETLOGLEVELRESPONSE = _descriptor.Descriptor(
  name='SetLogLevelResponse',
  full_name='koapy.grpc.SetLogLevelResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2338,
  serialized_end=2359,
)

_ARGUMENT.oneofs_by_name['value'].fields.append(
  _ARGUMENT.fields_by_name['string_value'])
_ARGUMENT.fields_by_name['string_value'].containing_oneof = _ARGUMENT.oneofs_by_name['value']
_ARGUMENT.oneofs_by_name['value'].fields.append(
  _ARGUMENT.fields_by_name['long_value'])
_ARGUMENT.fields_by_name['long_value'].containing_oneof = _ARGUMENT.oneofs_by_name['value']
_CALLREQUEST.fields_by_name['arguments'].message_type = _ARGUMENT
_RETURNVALUE.oneofs_by_name['value'].fields.append(
  _RETURNVALUE.fields_by_name['string_value'])
_RETURNVALUE.fields_by_name['string_value'].containing_oneof = _RETURNVALUE.oneofs_by_name['value']
_RETURNVALUE.oneofs_by_name['value'].fields.append(
  _RETURNVALUE.fields_by_name['long_value'])
_RETURNVALUE.fields_by_name['long_value'].containing_oneof = _RETURNVALUE.oneofs_by_name['value']
_CALLRESPONSE.fields_by_name['return_value'].message_type = _RETURNVALUE
_MULTIDATA.fields_by_name['values'].message_type = _REPEATEDSTRING
_LISTENRESPONSE.fields_by_name['arguments'].message_type = _ARGUMENT
_CUSTOMLISTENRESPONSE.fields_by_name['arguments'].message_type = _ARGUMENT
_CUSTOMLISTENRESPONSE.fields_by_name['single_data'].message_type = _SINGLEDATA
_CUSTOMLISTENRESPONSE.fields_by_name['multi_data'].message_type = _MULTIDATA
_CUSTOMCALLANDLISTENREQUEST.fields_by_name['call_request'].message_type = _CALLREQUEST
_CUSTOMCALLANDLISTENREQUEST.fields_by_name['listen_request'].message_type = _CUSTOMLISTENREQUEST
_CUSTOMCALLANDLISTENRESPONSE.fields_by_name['call_response'].message_type = _CALLRESPONSE
_CUSTOMCALLANDLISTENRESPONSE.fields_by_name['listen_response'].message_type = _CUSTOMLISTENRESPONSE
_CUSTOMCALLANDLISTENRESPONSE.oneofs_by_name['response'].fields.append(
  _CUSTOMCALLANDLISTENRESPONSE.fields_by_name['call_response'])
_CUSTOMCALLANDLISTENRESPONSE.fields_by_name['call_response'].containing_oneof = _CUSTOMCALLANDLISTENRESPONSE.oneofs_by_name['response']
_CUSTOMCALLANDLISTENRESPONSE.oneofs_by_name['response'].fields.append(
  _CUSTOMCALLANDLISTENRESPONSE.fields_by_name['listen_response'])
_CUSTOMCALLANDLISTENRESPONSE.fields_by_name['listen_response'].containing_oneof = _CUSTOMCALLANDLISTENRESPONSE.oneofs_by_name['response']
_TRANSACTIONSTOPCONDITION.fields_by_name['comparator'].enum_type = _TRANSACTIONSTOPCONDITIONCOMPARTOR
_TRANSACTIONREQUEST_INPUTSENTRY.containing_type = _TRANSACTIONREQUEST
_TRANSACTIONREQUEST.fields_by_name['inputs'].message_type = _TRANSACTIONREQUEST_INPUTSENTRY
_TRANSACTIONREQUEST.fields_by_name['output'].message_type = _TRANSACTIONSUBSCRIPTIONTARGET
_TRANSACTIONREQUEST.fields_by_name['stop_condition'].message_type = _TRANSACTIONSTOPCONDITION
_ORDERREQUEST.fields_by_name['target'].message_type = _ORDERSUBSCRIPTIONTARGET
_REALREQUEST.fields_by_name['flags'].message_type = _REALREQUESTFLAGS
DESCRIPTOR.message_types_by_name['Argument'] = _ARGUMENT
DESCRIPTOR.message_types_by_name['CallRequest'] = _CALLREQUEST
DESCRIPTOR.message_types_by_name['ReturnValue'] = _RETURNVALUE
DESCRIPTOR.message_types_by_name['CallResponse'] = _CALLRESPONSE
DESCRIPTOR.message_types_by_name['ListenRequest'] = _LISTENREQUEST
DESCRIPTOR.message_types_by_name['SingleData'] = _SINGLEDATA
DESCRIPTOR.message_types_by_name['RepeatedString'] = _REPEATEDSTRING
DESCRIPTOR.message_types_by_name['MultiData'] = _MULTIDATA
DESCRIPTOR.message_types_by_name['ListenResponse'] = _LISTENRESPONSE
DESCRIPTOR.message_types_by_name['StopListenRequest'] = _STOPLISTENREQUEST
DESCRIPTOR.message_types_by_name['StopListenResponse'] = _STOPLISTENRESPONSE
DESCRIPTOR.message_types_by_name['CustomListenRequest'] = _CUSTOMLISTENREQUEST
DESCRIPTOR.message_types_by_name['CustomListenResponse'] = _CUSTOMLISTENRESPONSE
DESCRIPTOR.message_types_by_name['CustomCallAndListenRequest'] = _CUSTOMCALLANDLISTENREQUEST
DESCRIPTOR.message_types_by_name['CustomCallAndListenResponse'] = _CUSTOMCALLANDLISTENRESPONSE
DESCRIPTOR.message_types_by_name['LoginRequest'] = _LOGINREQUEST
DESCRIPTOR.message_types_by_name['TransactionSubscriptionTarget'] = _TRANSACTIONSUBSCRIPTIONTARGET
DESCRIPTOR.message_types_by_name['TransactionStopCondition'] = _TRANSACTIONSTOPCONDITION
DESCRIPTOR.message_types_by_name['TransactionRequest'] = _TRANSACTIONREQUEST
DESCRIPTOR.message_types_by_name['OrderSubscriptionTarget'] = _ORDERSUBSCRIPTIONTARGET
DESCRIPTOR.message_types_by_name['OrderRequest'] = _ORDERREQUEST
DESCRIPTOR.message_types_by_name['RealRequestFlags'] = _REALREQUESTFLAGS
DESCRIPTOR.message_types_by_name['RealRequest'] = _REALREQUEST
DESCRIPTOR.message_types_by_name['SetLogLevelRequest'] = _SETLOGLEVELREQUEST
DESCRIPTOR.message_types_by_name['SetLogLevelResponse'] = _SETLOGLEVELRESPONSE
DESCRIPTOR.enum_types_by_name['TransactionStopConditionCompartor'] = _TRANSACTIONSTOPCONDITIONCOMPARTOR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Argument = _reflection.GeneratedProtocolMessageType('Argument', (_message.Message,), {
  'DESCRIPTOR' : _ARGUMENT,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.Argument)
  })
_sym_db.RegisterMessage(Argument)

CallRequest = _reflection.GeneratedProtocolMessageType('CallRequest', (_message.Message,), {
  'DESCRIPTOR' : _CALLREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.CallRequest)
  })
_sym_db.RegisterMessage(CallRequest)

ReturnValue = _reflection.GeneratedProtocolMessageType('ReturnValue', (_message.Message,), {
  'DESCRIPTOR' : _RETURNVALUE,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.ReturnValue)
  })
_sym_db.RegisterMessage(ReturnValue)

CallResponse = _reflection.GeneratedProtocolMessageType('CallResponse', (_message.Message,), {
  'DESCRIPTOR' : _CALLRESPONSE,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.CallResponse)
  })
_sym_db.RegisterMessage(CallResponse)

ListenRequest = _reflection.GeneratedProtocolMessageType('ListenRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTENREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.ListenRequest)
  })
_sym_db.RegisterMessage(ListenRequest)

SingleData = _reflection.GeneratedProtocolMessageType('SingleData', (_message.Message,), {
  'DESCRIPTOR' : _SINGLEDATA,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.SingleData)
  })
_sym_db.RegisterMessage(SingleData)

RepeatedString = _reflection.GeneratedProtocolMessageType('RepeatedString', (_message.Message,), {
  'DESCRIPTOR' : _REPEATEDSTRING,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.RepeatedString)
  })
_sym_db.RegisterMessage(RepeatedString)

MultiData = _reflection.GeneratedProtocolMessageType('MultiData', (_message.Message,), {
  'DESCRIPTOR' : _MULTIDATA,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.MultiData)
  })
_sym_db.RegisterMessage(MultiData)

ListenResponse = _reflection.GeneratedProtocolMessageType('ListenResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTENRESPONSE,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.ListenResponse)
  })
_sym_db.RegisterMessage(ListenResponse)

StopListenRequest = _reflection.GeneratedProtocolMessageType('StopListenRequest', (_message.Message,), {
  'DESCRIPTOR' : _STOPLISTENREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.StopListenRequest)
  })
_sym_db.RegisterMessage(StopListenRequest)

StopListenResponse = _reflection.GeneratedProtocolMessageType('StopListenResponse', (_message.Message,), {
  'DESCRIPTOR' : _STOPLISTENRESPONSE,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.StopListenResponse)
  })
_sym_db.RegisterMessage(StopListenResponse)

CustomListenRequest = _reflection.GeneratedProtocolMessageType('CustomListenRequest', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMLISTENREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.CustomListenRequest)
  })
_sym_db.RegisterMessage(CustomListenRequest)

CustomListenResponse = _reflection.GeneratedProtocolMessageType('CustomListenResponse', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMLISTENRESPONSE,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.CustomListenResponse)
  })
_sym_db.RegisterMessage(CustomListenResponse)

CustomCallAndListenRequest = _reflection.GeneratedProtocolMessageType('CustomCallAndListenRequest', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMCALLANDLISTENREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.CustomCallAndListenRequest)
  })
_sym_db.RegisterMessage(CustomCallAndListenRequest)

CustomCallAndListenResponse = _reflection.GeneratedProtocolMessageType('CustomCallAndListenResponse', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMCALLANDLISTENRESPONSE,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.CustomCallAndListenResponse)
  })
_sym_db.RegisterMessage(CustomCallAndListenResponse)

LoginRequest = _reflection.GeneratedProtocolMessageType('LoginRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOGINREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.LoginRequest)
  })
_sym_db.RegisterMessage(LoginRequest)

TransactionSubscriptionTarget = _reflection.GeneratedProtocolMessageType('TransactionSubscriptionTarget', (_message.Message,), {
  'DESCRIPTOR' : _TRANSACTIONSUBSCRIPTIONTARGET,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.TransactionSubscriptionTarget)
  })
_sym_db.RegisterMessage(TransactionSubscriptionTarget)

TransactionStopCondition = _reflection.GeneratedProtocolMessageType('TransactionStopCondition', (_message.Message,), {
  'DESCRIPTOR' : _TRANSACTIONSTOPCONDITION,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.TransactionStopCondition)
  })
_sym_db.RegisterMessage(TransactionStopCondition)

TransactionRequest = _reflection.GeneratedProtocolMessageType('TransactionRequest', (_message.Message,), {

  'InputsEntry' : _reflection.GeneratedProtocolMessageType('InputsEntry', (_message.Message,), {
    'DESCRIPTOR' : _TRANSACTIONREQUEST_INPUTSENTRY,
    '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
    # @@protoc_insertion_point(class_scope:koapy.grpc.TransactionRequest.InputsEntry)
    })
  ,
  'DESCRIPTOR' : _TRANSACTIONREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.TransactionRequest)
  })
_sym_db.RegisterMessage(TransactionRequest)
_sym_db.RegisterMessage(TransactionRequest.InputsEntry)

OrderSubscriptionTarget = _reflection.GeneratedProtocolMessageType('OrderSubscriptionTarget', (_message.Message,), {
  'DESCRIPTOR' : _ORDERSUBSCRIPTIONTARGET,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.OrderSubscriptionTarget)
  })
_sym_db.RegisterMessage(OrderSubscriptionTarget)

OrderRequest = _reflection.GeneratedProtocolMessageType('OrderRequest', (_message.Message,), {
  'DESCRIPTOR' : _ORDERREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.OrderRequest)
  })
_sym_db.RegisterMessage(OrderRequest)

RealRequestFlags = _reflection.GeneratedProtocolMessageType('RealRequestFlags', (_message.Message,), {
  'DESCRIPTOR' : _REALREQUESTFLAGS,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.RealRequestFlags)
  })
_sym_db.RegisterMessage(RealRequestFlags)

RealRequest = _reflection.GeneratedProtocolMessageType('RealRequest', (_message.Message,), {
  'DESCRIPTOR' : _REALREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.RealRequest)
  })
_sym_db.RegisterMessage(RealRequest)

SetLogLevelRequest = _reflection.GeneratedProtocolMessageType('SetLogLevelRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETLOGLEVELREQUEST,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.SetLogLevelRequest)
  })
_sym_db.RegisterMessage(SetLogLevelRequest)

SetLogLevelResponse = _reflection.GeneratedProtocolMessageType('SetLogLevelResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETLOGLEVELRESPONSE,
  '__module__' : 'koapy.grpc.KiwoomOpenApiService_pb2'
  # @@protoc_insertion_point(class_scope:koapy.grpc.SetLogLevelResponse)
  })
_sym_db.RegisterMessage(SetLogLevelResponse)


_TRANSACTIONREQUEST_INPUTSENTRY._options = None

_KIWOOMOPENAPISERVICE = _descriptor.ServiceDescriptor(
  name='KiwoomOpenApiService',
  full_name='koapy.grpc.KiwoomOpenApiService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=2522,
  serialized_end=3382,
  methods=[
  _descriptor.MethodDescriptor(
    name='Call',
    full_name='koapy.grpc.KiwoomOpenApiService.Call',
    index=0,
    containing_service=None,
    input_type=_CALLREQUEST,
    output_type=_CALLRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Listen',
    full_name='koapy.grpc.KiwoomOpenApiService.Listen',
    index=1,
    containing_service=None,
    input_type=_LISTENREQUEST,
    output_type=_CUSTOMLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StopListen',
    full_name='koapy.grpc.KiwoomOpenApiService.StopListen',
    index=2,
    containing_service=None,
    input_type=_STOPLISTENREQUEST,
    output_type=_STOPLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CustomListen',
    full_name='koapy.grpc.KiwoomOpenApiService.CustomListen',
    index=3,
    containing_service=None,
    input_type=_CUSTOMLISTENREQUEST,
    output_type=_CUSTOMLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CustomCallAndListen',
    full_name='koapy.grpc.KiwoomOpenApiService.CustomCallAndListen',
    index=4,
    containing_service=None,
    input_type=_CUSTOMCALLANDLISTENREQUEST,
    output_type=_CUSTOMCALLANDLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='LoginCall',
    full_name='koapy.grpc.KiwoomOpenApiService.LoginCall',
    index=5,
    containing_service=None,
    input_type=_LOGINREQUEST,
    output_type=_CUSTOMCALLANDLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='TransactionCall',
    full_name='koapy.grpc.KiwoomOpenApiService.TransactionCall',
    index=6,
    containing_service=None,
    input_type=_TRANSACTIONREQUEST,
    output_type=_CUSTOMCALLANDLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='OrderCall',
    full_name='koapy.grpc.KiwoomOpenApiService.OrderCall',
    index=7,
    containing_service=None,
    input_type=_ORDERREQUEST,
    output_type=_CUSTOMCALLANDLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RealCall',
    full_name='koapy.grpc.KiwoomOpenApiService.RealCall',
    index=8,
    containing_service=None,
    input_type=_REALREQUEST,
    output_type=_CUSTOMCALLANDLISTENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetLogLevel',
    full_name='koapy.grpc.KiwoomOpenApiService.SetLogLevel',
    index=9,
    containing_service=None,
    input_type=_SETLOGLEVELREQUEST,
    output_type=_SETLOGLEVELRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_KIWOOMOPENAPISERVICE)

DESCRIPTOR.services_by_name['KiwoomOpenApiService'] = _KIWOOMOPENAPISERVICE

# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python

"""Tests for `koapy` package."""

import re
import pytest

from click.testing import CliRunner

from koapy import cli

def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.cli)
    assert result.exit_code == 0
    assert 'Usage' in result.output
    help_result = runner.invoke(cli.cli, ['--help'])
    assert help_result.exit_code == 0
    assert re.search(r'-h, --help[ ]* Show this message and exit.', help_result.output) is not None
    assert 'serve' in help_result.output



import json
import requests

from discord import Webhook, RequestsWebhookAdapter

from koapy.utils.messaging.Messenger import Messenger
from koapy.config import config

class DiscordWebhookMessenger(Messenger):

    def __init__(self, url=None):
        self._url = url or config.get_string('koapy.utils.messaging.discord.webhook_url')
        assert self._url is not None and len(self._url) > 0
        self._webhook = Webhook.from_url(self._url, adapter=RequestsWebhookAdapter())

    def send_message(self, content):
        return self._webhook.send(content)

class DoItYourselfDiscordWebhookMessenger(Messenger):

    def __init__(self, url=None):
        self._url = url or config.get_string('koapy.utils.messaging.discord.webhook_url')
        assert self._url is not None and len(self._url) > 0

    def send_message(self, content):
        headers = {
            'Content-Type': 'application/json',
        }
        data = {
            'content': content,
        }
        data = json.dumps(data)
        response = requests.post(self._url, headers=headers, data=data, params={'wait': 'true'})
        return response



import threading

from koapy.grpc import KiwoomOpenApiService_pb2, KiwoomOpenApiService_pb2_grpc
from koapy.grpc.event.BaseKiwoomOpenApiEventHandler import BaseKiwoomOpenApiEventHandler
from koapy.grpc.event.KiwoomOpenApiEventHandler import KiwoomOpenApiAllEventHandler
from koapy.grpc.event.KiwoomOpenApiEventHandler import KiwoomOpenApiSomeEventHandler
from koapy.grpc.event.KiwoomOpenApiEventHandler import KiwoomOpenApiLoginEventHandler
from koapy.grpc.event.KiwoomOpenApiEventHandler import KiwoomOpenApiTrEventHandler
from koapy.grpc.event.KiwoomOpenApiEventHandler import KiwoomOpenApiOrderEventHandler
from koapy.grpc.event.KiwoomOpenApiEventHandler import KiwoomOpenApiRealEventHandler
from koapy.openapi.KiwoomOpenApiError import KiwoomOpenApiError

from koapy.utils.itertools import chunk
from koapy.utils.logging import set_loglevel

class KiwoomOpenApiServiceServicer(KiwoomOpenApiService_pb2_grpc.KiwoomOpenApiServiceServicer):

    _listen_id_to_handler = {}
    _listen_id_to_handler_lock = threading.RLock()

    def __init__(self, control):
        super().__init__()
        self._control = control

    @property
    def control(self):
        return self._control

    @classmethod
    def _convertArguments(cls, arguments):
        args = []
        for argument in arguments:
            if argument.HasField('string_value'):
                args.append(argument.string_value)
            elif argument.HasField('long_value'):
                args.append(argument.long_value)
        return args

    def Call(self, request, context):
        name = request.name
        arguments = self._convertArguments(request.arguments)
        function = getattr(self.control, name)
        result = function(*arguments)
        response = KiwoomOpenApiService_pb2.CallResponse()
        if isinstance(result, str):
            response.return_value.string_value = result # pylint: disable=no-member
        elif isinstance(result, int):
            response.return_value.long_value = result # pylint: disable=no-member
        elif result is None:
            pass
        else:
            raise TypeError('Unexpected return value type from server side dynamicCall(): %s' % type(result))
        return response

    def _RegisterHandler(self, handler_id, handler):
        with self._listen_id_to_handler_lock:
            self._UnregisterHandler(handler_id)
            self._listen_id_to_handler[handler_id] = handler

    def _UnregisterHandler(self, handler_id):
        with self._listen_id_to_handler_lock:
            if handler_id in self._listen_id_to_handler:
                self._listen_id_to_handler[handler_id].observer.on_completed()
                del self._listen_id_to_handler[handler_id]
                return True
        return False

    def Listen(self, request, context):
        handler = KiwoomOpenApiSomeEventHandler(self.control, request)
        self._RegisterHandler(request.id, handler)
        handler.add_callback(self._UnregisterHandler, request.id)
        with handler:
            for response in handler:
                yield response

    def StopListen(self, request, context):
        response = KiwoomOpenApiService_pb2.StopListenResponse()
        response.successful = self._UnregisterHandler(request.id)
        return response

    def CustomListen(self, request, context):
        code = request.code
        class_name = request.class_name
        if code and class_name:
            g = {}
            l = {}
            exec(code, g, l) # pylint: disable=exec-used
            handler = eval(class_name, g, l)(self.control, request) # pylint: disable=eval-used
            assert isinstance(handler, BaseKiwoomOpenApiEventHandler)
            with handler:
                for response in handler:
                    yield response
        else:
            handler = KiwoomOpenApiAllEventHandler(self.control)
            with handler:
                for response in handler:
                    yield response

    def CustomCallAndListen(self, request, context):
        name = request.name
        arguments = self._convertArguments(request.arguments)
        function = getattr(self.control, name)
        code = request.listen_request.code
        class_name = request.listen_request.class_name
        if code and class_name:
            g = {}
            l = {}
            exec(code, g, l) # pylint: disable=exec-used
            handler = eval(class_name, g, l)(self.control, request) # pylint: disable=eval-used
            assert isinstance(handler, BaseKiwoomOpenApiEventHandler)
        else:
            handler = KiwoomOpenApiAllEventHandler(self.control)
        with handler:
            result = function(*arguments)
            response = KiwoomOpenApiService_pb2.CustomCallAndListenResponse()
            if isinstance(result, str):
                response.call_response.return_value.string_value = result # pylint: disable=no-member
            elif isinstance(result, int):
                response.call_response.return_value.long_value = result # pylint: disable=no-member
            elif result is None:
                pass
            else:
                raise TypeError('Unexpected return value type from server side dynamicCall(): %s' % type(result))
            yield response
            for response in handler:
                yield response

    def LoginCall(self, request, context):
        with KiwoomOpenApiLoginEventHandler(self.control, request) as handler:
            KiwoomOpenApiError.try_or_raise(
                self.control.CommConnect())
            for response in handler:
                yield response

    def TransactionCall(self, request, context):
        rqname = request.request_name
        trcode = request.transaction_code
        prevnext = 0
        scrnno = request.screen_no
        inputs = request.inputs
        with KiwoomOpenApiTrEventHandler(self.control, request) as handler:
            handler.add_callback(self.control.DisconnectRealData, scrnno)
            for k, v in inputs.items():
                self.control.SetInputValue(k, v)
            KiwoomOpenApiError.try_or_raise(
                self.control.RateLimitedCommRqData(rqname, trcode, int(prevnext), scrnno, inputs))
            for response in handler:
                yield response

    def OrderCall(self, request, context):
        rqname = request.request_name
        scrnno = request.screen_no
        accno = request.account_no
        ordertype = request.order_type
        code = request.code
        qty = request.quantity
        price = request.price
        hogagb = request.quote_type
        orgorderno = request.original_order_no
        with KiwoomOpenApiOrderEventHandler(self.control, request) as handler:
            KiwoomOpenApiError.try_or_raise(
                self.control.SendOrder(rqname, scrnno, accno, ordertype, code, qty, price, hogagb, orgorderno))
            for response in handler:
                yield response

    def RealCall(self, request, context):
        code_lists = [';'.join(codes) for codes in chunk(request.code_list, 100)]
        screen_nos = [str(int(request.screen_no or '0001') + i).zfill(4) for i in range(len(code_lists))]
        fid_list = ';'.join([str(fid) for fid in request.fid_list])
        real_type = request.real_type or '0'
        with KiwoomOpenApiRealEventHandler(self.control, request) as handler:
            for screen_no, code_list in zip(screen_nos, code_lists):
                handler.add_callback(self.control.DisconnectRealData, screen_no)
                handler.add_callback(self.control.SetRealRemove, screen_no, 'ALL')
                KiwoomOpenApiError.try_or_raise(
                    self.control.SetRealReg(screen_no, code_list, fid_list, real_type))
            for response in handler:
                yield response

    def SetLogLevel(self, request, context):
        level = request.level
        logger = request.logger
        set_loglevel(level, logger)
        response = KiwoomOpenApiService_pb2.SetLogLevelResponse()
        return response



class KiwoomOpenApiError(Exception):
    OP_ERR_NONE = 0
    OP_ERR_FAIL = -10
    OP_ERR_COND_NOTFOUND = -11
    OP_ERR_COND_MISMATCH = -12
    OP_ERR_COND_OVERFLOW = -13
    OP_ERR_LOGIN = -100
    OP_ERR_CONNECT = -101
    OP_ERR_VERSION = -102
    OP_ERR_FIREWALL = -103
    OP_ERR_MEMORY = -104
    OP_ERR_INPUT = -105
    OP_ERR_SOCKET_CLOSED = -106
    OP_ERR_SISE_OVERFLOW = -200
    OP_ERR_RQ_STRUCT_FAIL = -201
    OP_ERR_RQ_STRING_FAIL = -202
    OP_ERR_NO_DATA = -203
    OP_ERR_OVER_MAX_DATA = -204
    OP_ERR_DATA_RCV_FAIL = -205
    OP_ERR_OVER_MAX_FID = -206
    OP_ERR_REAL_CANCEL = -207
    OP_ERR_ORD_WRONG_INPUT = -300
    OP_ERR_ORD_WRONG_ACCTNO = -301
    OP_ERR_OTHER_ACC_USE = -302
    OP_ERR_MIS_2BILL_EXC = -303
    OP_ERR_MIS_5BILL_EXC = -304
    OP_ERR_MIS_1PER_EXC = -305
    OP_ERR_MIS_3PER_EXC = -306
    OP_ERR_SEND_FAIL = -307
    OP_ERR_ORD_OVERFLOW = -308
    OP_ERR_ORD_OVERFLOW2 = -311
    OP_ERR_MIS_300CNT_EXC = -309
    OP_ERR_MIS_500CNT_EXC = -310
    OP_ERR_ORD_WRONG_ACCTINFO = -340
    OP_ERR_ORD_SYMCODE_EMPTY = -500

    MSG_ERR_NONE = '정상처리'
    MSG_ERR_FAIL = '실패'
    MSG_ERR_COND_NOTFOUND = '조건번호 없음'
    MSG_ERR_COND_MISMATCH = '조건번호와 조건식 틀림'
    MSG_ERR_COND_OVERFLOW = '조건검색 조회요청 초과'
    MSG_ERR_LOGIN = '사용자정보 교환 실패'
    MSG_ERR_CONNECT = '서버접속 실패'
    MSG_ERR_VERSION = '버전처리 실패'
    MSG_ERR_FIREWALL = '개인방화벽 실패'
    MSG_ERR_MEMORY = '메모리보호 실패'
    MSG_ERR_INPUT = '함수입력값 오류'
    MSG_ERR_SOCKET_CLOSED = '통신 연결종료'
    MSG_ERR_SISE_OVERFLOW = '시세조회 과부하'
    MSG_ERR_RQ_STRUCT_FAIL = '전문작성 초기화 실패'
    MSG_ERR_RQ_STRING_FAIL = '전문작성 입력값 오류'
    MSG_ERR_NO_DATA = '데이터 없음'
    MSG_ERR_OVER_MAX_DATA = '조회 가능한 종목수 초과'
    MSG_ERR_DATA_RCV_FAIL = '데이터수신 실패'
    MSG_ERR_OVER_MAX_FID = '조회 가능한 FID수 초과'
    MSG_ERR_REAL_CANCEL = '실시간 해제 오류'
    MSG_ERR_ORD_WRONG_INPUT = '입력값 오류'
    MSG_ERR_ORD_WRONG_ACCTNO = '계좌 비밀번호 없음'
    MSG_ERR_OTHER_ACC_USE = '타인계좌사용 오류'
    MSG_ERR_MIS_2BILL_EXC = '주문가격이 20억원을 초과'
    MSG_ERR_MIS_5BILL_EXC = '주문가격이 50억원을 초과'
    MSG_ERR_MIS_1PER_EXC = '주문수량이 총발행주수의 1%초과오류'
    MSG_ERR_MIS_3PER_EXC = '주문수량이 총발행주수의 3%초과오류'
    MSG_ERR_SEND_FAIL = '주문전송 실패'
    MSG_ERR_ORD_OVERFLOW = '주문전송 과부하'
    MSG_ERR_ORD_OVERFLOW2 = '주문전송 과부하'
    MSG_ERR_MIS_300CNT_EXC = '주문수량 300계약 초과'
    MSG_ERR_MIS_500CNT_EXC = '주문수량 500계약 초과'
    MSG_ERR_ORD_WRONG_ACCTINFO = '계좌정보없음'
    MSG_ERR_ORD_SYMCODE_EMPTY = '종목코드없음'

    ERROR_MESSAGE_BY_CODE = {
        OP_ERR_NONE: MSG_ERR_NONE,
        OP_ERR_FAIL: MSG_ERR_FAIL,
        OP_ERR_COND_NOTFOUND: MSG_ERR_COND_NOTFOUND,
        OP_ERR_COND_MISMATCH: MSG_ERR_COND_MISMATCH,
        OP_ERR_COND_OVERFLOW: MSG_ERR_COND_OVERFLOW,
        OP_ERR_LOGIN: MSG_ERR_LOGIN,
        OP_ERR_CONNECT: MSG_ERR_CONNECT,
        OP_ERR_VERSION: MSG_ERR_VERSION,
        OP_ERR_FIREWALL: MSG_ERR_FIREWALL,
        OP_ERR_MEMORY: MSG_ERR_MEMORY,
        OP_ERR_INPUT: MSG_ERR_INPUT,
        OP_ERR_SOCKET_CLOSED: MSG_ERR_SOCKET_CLOSED,
        OP_ERR_SISE_OVERFLOW: MSG_ERR_SISE_OVERFLOW,
        OP_ERR_RQ_STRUCT_FAIL: MSG_ERR_RQ_STRUCT_FAIL,
        OP_ERR_RQ_STRING_FAIL: MSG_ERR_RQ_STRING_FAIL,
        OP_ERR_NO_DATA: MSG_ERR_NO_DATA,
        OP_ERR_OVER_MAX_DATA: MSG_ERR_OVER_MAX_DATA,
        OP_ERR_DATA_RCV_FAIL: MSG_ERR_DATA_RCV_FAIL,
        OP_ERR_OVER_MAX_FID: MSG_ERR_OVER_MAX_FID,
        OP_ERR_REAL_CANCEL: MSG_ERR_REAL_CANCEL,
        OP_ERR_ORD_WRONG_INPUT: MSG_ERR_ORD_WRONG_INPUT,
        OP_ERR_ORD_WRONG_ACCTNO: MSG_ERR_ORD_WRONG_ACCTNO,
        OP_ERR_OTHER_ACC_USE: MSG_ERR_OTHER_ACC_USE,
        OP_ERR_MIS_2BILL_EXC: MSG_ERR_MIS_2BILL_EXC,
        OP_ERR_MIS_5BILL_EXC: MSG_ERR_MIS_5BILL_EXC,
        OP_ERR_MIS_1PER_EXC: MSG_ERR_MIS_1PER_EXC,
        OP_ERR_MIS_3PER_EXC: MSG_ERR_MIS_3PER_EXC,
        OP_ERR_SEND_FAIL: MSG_ERR_SEND_FAIL,
        OP_ERR_ORD_OVERFLOW: MSG_ERR_ORD_OVERFLOW,
        OP_ERR_ORD_OVERFLOW2: MSG_ERR_ORD_OVERFLOW2,
        OP_ERR_MIS_300CNT_EXC: MSG_ERR_MIS_300CNT_EXC,
        OP_ERR_MIS_500CNT_EXC: MSG_ERR_MIS_500CNT_EXC,
        OP_ERR_ORD_WRONG_ACCTINFO: MSG_ERR_ORD_WRONG_ACCTINFO,
        OP_ERR_ORD_SYMCODE_EMPTY: MSG_ERR_ORD_SYMCODE_EMPTY,
    }

    ERROR_CODES = list(ERROR_MESSAGE_BY_CODE.keys())
    ERROR_MESSAGES = list(ERROR_MESSAGE_BY_CODE.values())

    @classmethod
    def get_error_message_by_code(cls, code):
        return cls.ERROR_MESSAGE_BY_CODE.get(code)

    @classmethod
    def try_or_raise(cls, code_or_function, *args, **kwargs):
        if callable(code_or_function):
            code = code_or_function(*args, **kwargs)
        elif isinstance(code_or_function, int):
            code = code_or_function
        else:
            raise ValueError("Invalid argument for code_or_function: %s" % code_or_function)
        if code < 0:
            raise cls(code)
        return code

    def __init__(self, code, message=None):
        if message is None:
            message = self.get_error_message_by_code(code)

        self._code = code
        self._message = message

        super().__init__(self._message)

    def __str__(self):
        return self._message

    def __repr__(self):
        return '%s(%d, %r)' % (self.__class__.__name__, self._code, self._message)

    @property
    def code(self):
        return self._code

    @property
    def message(self):
        return self._message



import logging
import subprocess

from koapy.grpc.KiwoomOpenApiServiceClient import KiwoomOpenApiServiceClient

from koapy.config import config
from koapy.utils.networking import get_free_localhost_port
from koapy.utils.logging import verbosity_to_loglevel, set_loglevel

class KiwoomOpenApiContext:

    def __init__(self, port=None, client_check_timeout=None, verbosity=None, log_level=None):
        self._port = port or config.get('koapy.grpc.port') or get_free_localhost_port()
        self._client_check_timeout = client_check_timeout
        self._verbosity = verbosity
        self._log_level = log_level or verbosity_to_loglevel(self._verbosity)
        set_loglevel(self._log_level)
        self._server_proc_args = [
            'python', '-m', 'koapy.pyqt5.tools.start_tray_application',
            '--port', str(self._port)]
        if self._verbosity:
            self._server_proc_args.append('-' + 'v' * self._verbosity)
        self._server_proc = None
        self._server_proc_terminate_timeout = config.get_int('koapy.grpc.context.server.terminate.timeout', 10)
        self._client = KiwoomOpenApiServiceClient(port=self._port)
        logging.debug('Testing if client is ready...')
        if not self._client.is_ready(client_check_timeout):
            logging.debug('Client is not ready, creating a new server')
            self._server_proc = subprocess.Popen(self._server_proc_args)
            assert self._client.is_ready()
            self._stub = self._client.get_stub()
        else:
            logging.debug('Client is ready, using existing server')
            self._stub = self._client.get_stub()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def get_stub(self):
        return self._stub

    def close(self):
        self._client.close()
        if self._server_proc is not None:
            self._server_proc.terminate() # maybe soft termination available via grpc? rather than hard termination
            self._server_proc.wait(self._server_proc_terminate_timeout)
            self._server_proc = None

    def __getattr__(self, name):
        return getattr(self._stub, name)



import atexit

from queue import Queue, Empty
from koapy.grpc.observer.Observer import Observer

class QueueBasedIterableObserver(Observer):

    _queue_get_timeout = 2

    def __init__(self, queue=None):
        if queue is None:
            queue = Queue()

        self._queue = queue
        self._sentinel = object()

        atexit.register(self._queue.put, self._sentinel)

    def on_next(self, value):
        self._queue.put(value)

    def on_error(self, error):
        self._queue.put(error)

    def on_completed(self):
        self._queue.put(self._sentinel)

    def __iter__(self):
        while True:
            try:
                value = self._queue.get(True, self._queue_get_timeout)
            except Empty:
                pass
            else:
                if value == self._sentinel:
                    self._queue.task_done()
                    break
                if isinstance(value, Exception):
                    self._queue.task_done()
                    raise value
                yield value
                self._queue.task_done()



import os
import io
import json
import zipfile
import contextlib

from koapy.utils.serialization import JsonSerializable

class TrInfo(JsonSerializable):

    _TRINFO_BY_CODE_DUMP_FILENAME = 'trinfo_by_code.json'
    _TRINFO_BY_CODE = {}

    class Field(JsonSerializable):

        __outer_class__ = None

        def __init__(self, name=None, start=None, offset=None, fid=None):
            self.name = name
            self.start = start
            self.offset = offset
            self.fid = fid

        def __repr__(self):
            return '%s.%s(%r, %r, %r, %r)' % (
                self.__outer_class__.__name__,
                self.__class__.__name__,
                self.name,
                self.start,
                self.offset,
                self.fid)

    def __init__(self, tr_code=None, name=None, tr_name=None, tr_names_svr=None, tr_type=None, gfid=None,
                 inputs=None, single_outputs_name=None, single_outputs=None, multi_outputs_name=None, multi_outputs=None):
        self.tr_code = tr_code
        self.name = name
        self.tr_name = tr_name
        self.tr_names_svr = tr_names_svr
        self.tr_type = tr_type
        self.gfid = gfid
        self.inputs = inputs
        self.single_outputs_name = single_outputs_name
        self.single_outputs = single_outputs
        self.multi_outputs_name = multi_outputs_name
        self.multi_outputs = multi_outputs

    def __repr__(self):
        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % (
            self.__class__.__name__,
            self.tr_code,
            self.name,
            self.tr_name,
            self.tr_names_svr,
            self.tr_type,
            self.gfid,
            self.inputs,
            self.single_outputs_name,
            self.single_outputs,
            self.multi_outputs_name,
            self.multi_outputs
        )

    def to_dict(self):
        dic = dict(self.__dict__)
        for attr in dic:
            if isinstance(dic[attr], list):
                dic[attr] = [field.to_dict() for field in dic[attr]]
        return dic

    @classmethod
    def from_dict(cls, dic):
        output = cls()
        for name in output.__dict__:
            value = dic.get(name)
            if isinstance(value, list):
                fields = [cls.Field.from_dict(d) for d in value]
                setattr(output, name, fields)
            else:
                setattr(output, name, value)
        return output

    def get_input_names(self):
        return [input_.name for input_ in self.inputs]

    def get_single_output_names(self):
        return [output.name for output in self.single_outputs]

    def get_multi_output_names(self):
        return [output.name for output in self.multi_outputs]

    @classmethod
    def get_trinfo_by_code(cls, trcode):
        return cls._TRINFO_BY_CODE.get(trcode.lower())

    @classmethod
    def from_encfile(cls, f, tr_code=''):
        with contextlib.ExitStack() as stack:
            if isinstance(f, str):
                tr_code = os.path.splitext(f.lower())[0]
                f = stack.enter_context(open(f))

            lines = iter(f)
            lines = map(lambda line: line.rstrip('\n'), lines)
            lines = filter(lambda line: len(line.strip()) > 0, lines)

            single_outputs_name = ''
            single_outputs = []
            multi_outputs_name = ''
            multi_outputs = []

            tr_names_svr = ''
            gfid = ''

            line = next(lines)
            assert line == '[TRINFO]'
            line = next(lines)
            assert line.startswith('TRName=')
            tr_name = line.split('=', 2)[1]
            line = next(lines)
            if line.startswith('TRNameSVR='):
                tr_names_svr = line.split('=', 2)[1]
                line = next(lines)
            assert line.startswith('TRType=')
            tr_type = line.split('=', 2)[1]
            line = next(lines)
            if line.startswith('GFID='):
                gfid = line.split('=', 2)[1]
                line = next(lines)
            assert line == '[INPUT]'
            line = next(lines)
            assert line.startswith('@START_')
            tr_name_readable = line.split('_', 2)[1].split('=', 2)[0]
            line = next(lines)
            inputs = []
            while not line.startswith('@END_'):
                input_name, triple = [item.strip() for item in line.strip().split('=', 2)]
                triple = [item.strip() for item in triple.split(',')]
                start, offset, fid = [int(item) for item in triple]
                inputs.append(cls.Field(input_name, start, offset, fid))
                line = next(lines)
            line = next(lines)
            assert line == '[OUTPUT]'
            line = next(lines)
            assert line.startswith('@START_')
            single_outputs_name = line.split('_', 2)[1].split('=', 2)[0]
            line = next(lines)
            while not line.startswith('@END_'):
                output_name, triple = [item.strip() for item in line.strip().split('=', 2)]
                triple = [item.strip() for item in triple.split(',')]
                start, offset, fid = [int(item) if item else 0 for item in triple]
                single_outputs.append(cls.Field(output_name, start, offset, fid))
                line = next(lines)
            try:
                line = next(lines)
            except StopIteration:
                pass
            else:
                if line.startswith('@START_'):
                    multi_outputs_name = line.split('_', 2)[1].split('=', 2)[0]
                    line = next(lines)
                    while not line.startswith('@END_'):
                        output_name, triple = [item.strip() for item in line.strip().split('=', 2)]
                        triple = [item.strip() for item in triple.split(',')]
                        start, offset, fid = [int(item) if item else 0 for item in triple]
                        multi_outputs.append(cls.Field(output_name, start, offset, fid))
                        line = next(lines)
            return cls(tr_code, tr_name_readable, tr_name, tr_names_svr, tr_type, gfid,
                       inputs, single_outputs_name, single_outputs, multi_outputs_name, multi_outputs)

    @classmethod
    def infos_from_data_dir(cls, data_dir=None, encoding='euc-kr'):
        if data_dir is None:
            data_dir = r'C:\OpenAPI\data'
        enc_filenames = [filename.lower() for filename in os.listdir(data_dir)]
        enc_filenames = [filename for filename in enc_filenames if filename.startswith('o') and filename.endswith('.enc')]
        results = []
        for filename in enc_filenames:
            with zipfile.ZipFile(os.path.join(data_dir, filename)) as z:
                for info in z.infolist():
                    inner_filename = info.filename
                    tr_code = os.path.splitext(inner_filename.lower())[0]
                    with z.open(info) as b:
                        with io.TextIOWrapper(b, encoding=encoding) as f:
                            results.append(cls.from_encfile(f, tr_code))
        return results

    @classmethod
    def trinfo_by_name_from_data_dir(cls, data_dir=None):
        infos = cls.infos_from_data_dir(data_dir)
        result = {info.tr_code:info for info in infos}
        return result

    @classmethod
    def dump_trinfo_by_name(cls, dump_file=None):
        with contextlib.ExitStack() as stack:
            if dump_file is None:
                dump_file = os.path.join(os.path.dirname(__file__), 'data', cls._TRINFO_BY_CODE_DUMP_FILENAME)
            if isinstance(dump_file, str):
                dump_file = stack.enter_context(open(dump_file, 'w'))
            result = cls.trinfo_by_name_from_data_dir()
            for tr_code in result:
                result[tr_code] = result[tr_code].to_dict()
            return json.dump(result, dump_file)

    @classmethod
    def trinfo_by_name_from_dump_file(cls, dump_file=None):
        with contextlib.ExitStack() as stack:
            if dump_file is None:
                dump_file = os.path.join(os.path.dirname(__file__), 'data', cls._TRINFO_BY_CODE_DUMP_FILENAME)
            if isinstance(dump_file, str) and os.path.exists(dump_file):
                dump_file = stack.enter_context(open(dump_file, 'r'))
                result = json.load(dump_file)
                for tr_code in result:
                    result[tr_code] = cls.from_dict(result[tr_code])
                return result
        return {}

    @classmethod
    def load_from_dump_file(cls, dump_file=None):
        cls._TRINFO_BY_CODE = cls.trinfo_by_name_from_dump_file(dump_file)

TrInfo.Field.__outer_class__ = TrInfo

TrInfo.load_from_dump_file()

def main():
    TrInfo.dump_trinfo_by_name()

def infer_fids_by_tr_outputs():
    infos = TrInfo.infos_from_data_dir()
    fields = []
    for info in infos:
        for field in info.single_outputs:
            fields.append(field)
        for field in info.multi_outputs:
            fields.append(field)
    pairs = [[field.fid, field.name] for field in fields if field.fid != -1]
    import string # pylint: disable=import-outside-toplevel
    for pair in pairs:
        if pair[1].startswith('풋_'):
            pair[1] = pair[1][2:]
        if not pair[1][0] in string.ascii_letters and pair[1][-1] in 'ns':
            pair[1] = pair[1][:-1]
        pair[1] = pair[1].upper()
    pairs = [tuple(pair) for pair in pairs]
    pairs = list(set(pairs))
    pairs = sorted(pairs, key=lambda item: item[0])
    import pandas as pd # pylint: disable=import-outside-toplevel
    df = pd.DataFrame.from_records(pairs)
    df.to_excel('fid.xlsx', header=False, index=False)

if __name__ == '__main__':
    main()



import re
import logging
import datetime

import numpy as np
import pandas as pd

from koapy.grpc import KiwoomOpenApiService_pb2
from koapy.grpc.KiwoomOpenApiServiceClientSideDynamicCallable import KiwoomOpenApiServiceClientSideDynamicCallable
from koapy.openapi.KiwoomOpenApiError import KiwoomOpenApiError
from koapy.openapi.RealType import RealType

class KiwoomOpenApiServiceClientStubCoreWrapper:

    def __init__(self, stub):
        self._stub = stub

    def __getattr__(self, name):
        return KiwoomOpenApiServiceClientSideDynamicCallable(self._stub, name)

    def LoginCall(self):
        request = KiwoomOpenApiService_pb2.LoginRequest()
        for response in self._stub.LoginCall(request):
            errcode = response.listen_response.arguments[0].long_value
        return errcode

    def TransactionCall(self, rqname, trcode, scrnno, inputs, stop_condition=None):
        request = KiwoomOpenApiService_pb2.TransactionRequest()
        request.request_name = rqname
        request.transaction_code = trcode
        request.screen_no = scrnno
        for k, v in inputs.items():
            request.inputs[k] = v # pylint: disable=no-member
        if stop_condition:
            request.stop_condition.name = stop_condition.get('name') # pylint: disable=no-member
            request.stop_condition.value = str(stop_condition.get('value')) # pylint: disable=no-member
            request.stop_condition.comparator = { # pylint: disable=no-member
                '<=': KiwoomOpenApiService_pb2.TransactionStopConditionCompartor.LESS_THAN_OR_EQUAL_TO,
                '<': KiwoomOpenApiService_pb2.TransactionStopConditionCompartor.LESS_THAN,
                '>=': KiwoomOpenApiService_pb2.TransactionStopConditionCompartor.GREATER_THAN_OR_EQUAL_TO,
                '>': KiwoomOpenApiService_pb2.TransactionStopConditionCompartor.GREATER_THAN,
                '==': KiwoomOpenApiService_pb2.TransactionStopConditionCompartor.EQUAL_TO,
                '!=': KiwoomOpenApiService_pb2.TransactionStopConditionCompartor.NOT_EQUAL_TO,
            }.get(stop_condition.get('comparator', '<='))
        return self._stub.TransactionCall(request)

    def OrderCall(self, rqname, scrnno, account, order_type, code, quantity, price, quote_type, original_order_no=None):
        """
        [거래구분]
          모의투자에서는 지정가 주문과 시장가 주문만 가능합니다.

          00 : 지정가
          03 : 시장가
          05 : 조건부지정가
          06 : 최유리지정가
          07 : 최우선지정가
          10 : 지정가IOC
          13 : 시장가IOC
          16 : 최유리IOC
          20 : 지정가FOK
          23 : 시장가FOK
          26 : 최유리FOK
          61 : 장전시간외종가
          62 : 시간외단일가매매
          81 : 장후시간외종가

        [주문유형]
          1:신규매수, 2:신규매도 3:매수취소, 4:매도취소, 5:매수정정, 6:매도정정
        """
        request = KiwoomOpenApiService_pb2.OrderRequest()
        request.request_name = rqname
        request.screen_no = scrnno
        request.account_no = account
        request.order_type = int(order_type) if order_type else 0
        request.code = code
        request.quantity = int(quantity) if quantity else 0
        request.price = int(price) if price else 0
        request.quote_type = quote_type
        request.original_order_no = '' if original_order_no is None else original_order_no
        return self._stub.OrderCall(request)

    def RealCall(self, scrnno, codes, fids, realtype=None, infer_fids=False, readable_names=False, fast_parse=False):
        request = KiwoomOpenApiService_pb2.RealRequest()
        fids = [int(fid) for fid in fids]
        if realtype is None:
            realtype = '0'
        request.screen_no = scrnno
        request.code_list.extend(codes) # pylint: disable=no-member
        request.fid_list.extend(fids) # pylint: disable=no-member
        request.real_type = realtype
        request.flags.infer_fids = infer_fids # pylint: disable=no-member
        request.flags.readable_names = readable_names # pylint: disable=no-member
        request.flags.fast_parse = fast_parse # pylint: disable=no-member
        return self._stub.RealCall(request)
    
    def SetLogLevel(self, level, logger=''):
        request = KiwoomOpenApiService_pb2.SetLogLevelRequest()
        request.level = level
        request.logger = logger
        return self._stub.SetLogLevel(request)


class KiwoomOpenApiServiceClientStubWrapper(KiwoomOpenApiServiceClientStubCoreWrapper):

    def EnsureConnected(self):
        errcode = 0
        if self.GetConnectState() == 0:
            errcode = self.LoginCall()
        if errcode < 0:
            message = 'Failed to connect'
            korean_message = KiwoomOpenApiError.get_error_message_by_code(errcode)
            if korean_message is not None:
                message += ', Korean error message: %s' % korean_message
            logging.warning(message)
        return errcode

    def GetCodeListByMarketAsList(self, market):
        market = str(market)
        result = self.GetCodeListByMarket(market).rstrip(';')
        result = result.split(';') if result else []
        return result

    def GetNameListByMarketAsList(self, market):
        codes = self.GetCodeListByMarketAsList(market)
        names = [self.GetMasterCodeName(code) for code in codes]
        return names

    def GetAccountListAsList(self):
        accounts = self.GetLoginInfo('ACCLIST').rstrip(';')
        accounts = accounts.split(';') if accounts else []
        return accounts

    def GetMasterStockStateAsList(self, code):
        states = self.GetMasterStockState(code).strip()
        states = states.split('|') if states else []
        return states

    def _RemoveLeadingZerosForNumber(self, value, width=None):
        remove = False
        if width is None:
            remove = True
        elif isinstance(width, int) and len(value) == width:
            remove = True
        elif hasattr(width, '__iter__') and len(value) in width:
            remove = True
        if remove:
            return re.sub(r'^\s*([+-]?)[0]+([0-9]+(.[0-9]+)?)\s*$', r'\1\2', value)
        return value

    def _RemoveLeadingZerosForNumbersInValues(self, values, width=None):
        return [self._RemoveLeadingZerosForNumber(value, width) for value in values]

    def GetCommonCodeList(self):
        """
        [시장구분값]
          0 : 장내
          10 : 코스닥
          3 : ELW
          8 : ETF
          50 : KONEX
          4 : 뮤추얼펀드
          5 : 신주인수권
          6 : 리츠
          9 : 하이얼펀드
          30 : K-OTC
        """

        # 기본 코드 리스트
        codes = self.GetCodeListByMarketAsList('0')

        # 장내 시장에서 ETN 이 섞여 있는데 시장구분값으로 뺄 수가 없어서 이름을 보고 대충 제외
        names = [self.GetMasterCodeName(code) for code in codes]
        etn_suffixes = ['ETN', 'ETN(H)', 'ETN B', 'ETN(H) B']
        is_not_etn_name = [not any(name.endswith(suffix) for suffix in etn_suffixes) for name in names]
        codes = np.array(codes)[is_not_etn_name].tolist()

        # 코드값 기준 제외 준비
        codes = set(codes)

        # 우선주 구분은 기본정보에서 유통주식 수량이 확인되지 않는 주식들
        # 그 중에 ~3호 등 배 관련 종목은 우선주가 아니고 PER 같은 값이 있음
        # 종목코드의 마지막 자리가 숫자가 아닌 K,L 등 인 것으로 구분할 수도 있는데 모두 그런건 아님
        # 당장은 미리  구분해놓은 값들을 참고해 제외하는 식으로
        preferred_stock_codes = []
        codes = codes - set(preferred_stock_codes)

        # 나머지는 혹시나 겹치는 애들이 나올 수 있는 시장에서 코드기준 제외
        codes = codes - set(self.GetCodeListByMarketAsList('10'))
        codes = codes - set(self.GetCodeListByMarketAsList('8'))
        codes = codes - set(self.GetCodeListByMarketAsList('4'))
        codes = codes - set(self.GetCodeListByMarketAsList('6'))

        # 정렬된 리스트 형태로 제공
        codes = sorted(list(codes))

        return codes

    def GetKosdaqCodeList(self):
        codes = self.GetCodeListByMarketAsList('10')
        codes = sorted(codes)
        return codes

    def IsSuspended(self, code):
        return '거래정지' in self.GetMasterStockStateAsList(code)

    def IsInSupervision(self, code):
        return '관리종목' in self.GetMasterStockStateAsList(code)

    def IsInSurveillance(self, code):
        return '감리종목' in self.GetMasterStockStateAsList(code)

    def GetStockInfoAsDataFrame(self, codes=None, rqname=None, scrnno=None):
        """
        [화면번호]
         화면번호는 서버에 조회나 주문등 필요한 기능을 요청할때 이를 구별하기 위한 키값으로 이해하시면 됩니다.
         0000(혹은 0)을 제외한 임의의 숫자를 사용하시면 되는데 갯수가 200개로 한정되어 있기 때문에 이 갯수를 넘지 않도록 관리하셔야 합니다.
         만약 사용하는 화면번호가 200개를 넘는 경우 조회결과나 주문결과에 다른 데이터가 섞이거나 원하지 않는 결과를 나타날 수 있습니다.
        """
        if codes is None:
            codes = self.GetCommonCodeList()
        if rqname is None:
            rqname = '주식기본정보요청'
        if scrnno is None:
            scrnno = '0291'
        trcode = 'opt10001'
        codes_len = len(codes)
        columns = []
        records = []
        for i, code in enumerate(codes):
            logging.debug('Getting basic info for code: %s (%d/%d)', code, i+1, codes_len)
            inputs = {'종목코드': code}
            for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
                if not columns:
                    columns = response.listen_response.single_data.names
                records.append(response.listen_response.single_data.values)
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetDailyStockDataAsDataFrame(self, code, start_date=None, end_date=None, rqname=None, scrnno=None):
        """
        [수정주가구분] 1:유상증자, 2:무상증자, 4:배당락, 8:액면분할, 16:액면병합, 32:기업합병, 64:감자, 256:권리락
        """
        if start_date is None:
            now = datetime.datetime.now()
            start_date = now
            if now < now.replace(hour=15, minute=30):
                start_date -= datetime.timedelta(days=1)
        date_format = '%Y%m%d'
        if isinstance(start_date, datetime.datetime):
            start_date = start_date.strftime(date_format)
        if end_date is not None:
            if isinstance(end_date, datetime.datetime):
                end_date = end_date.strftime(date_format)
            stop_condition = {
                'name': '일자',
                'value': end_date,
            }
        else:
            stop_condition = None
        if rqname is None:
            rqname = '주식일봉차트조회요청'
        if scrnno is None:
            scrnno = '0613'
        trcode = 'opt10081'
        inputs = {
            '종목코드': code,
            '기준일자': start_date,
            '수정주가구분': '1',
        }
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs, stop_condition=stop_condition):
            if not columns:
                columns = list(response.listen_response.multi_data.names)
            for values in response.listen_response.multi_data.values:
                records.append(values.values)
            date_index = columns.index('일자')
            if len(response.listen_response.multi_data.values) > 0:
                from_date = response.listen_response.multi_data.values[0].values[date_index]
                to_date = response.listen_response.multi_data.values[-1].values[date_index]
                from_date = datetime.datetime.strptime(from_date, date_format)
                to_date = datetime.datetime.strptime(to_date, date_format)
                logging.debug('Received data from %s to %s for code %s', from_date, to_date, code)
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetMinuteStockDataAsDataFrame(self, code, interval=None, start_date=None, end_date=None, rqname=None, scrnno=None):
        """
        [수정주가구분] 1:유상증자, 2:무상증자, 4:배당락, 8:액면분할, 16:액면병합, 32:기업합병, 64:감자, 256:권리락
        """
        if interval is None:
            interval = 15
        if interval is not None:
            interval = str(interval)
        if start_date is None:
            now = datetime.datetime.now()
            start_date = now
            if now < now.replace(hour=15, minute=30):
                start_date -= datetime.timedelta(days=1)
        date_format = '%Y%m%d%H%M%S'
        if isinstance(start_date, datetime.datetime):
            start_date = start_date.strftime(date_format)
        if end_date is not None:
            if isinstance(end_date, datetime.datetime):
                end_date = end_date.strftime(date_format)
            stop_condition = {
                'name': '체결시간',
                'value': end_date,
            }
        else:
            stop_condition = None
        if rqname is None:
            rqname = '주식분봉차트조회요청'
        if scrnno is None:
            scrnno = '0613'
        trcode = 'opt10080'
        inputs = {
            '종목코드': code,
            '틱범위': interval,
            '수정주가구분': '1',
        }
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs, stop_condition=stop_condition):
            if not columns:
                columns = list(response.listen_response.multi_data.names)
            for values in response.listen_response.multi_data.values:
                records.append(values.values)
            date_index = columns.index('체결시간')
            if len(response.listen_response.multi_data.values) > 0:
                from_date = response.listen_response.multi_data.values[0].values[date_index]
                to_date = response.listen_response.multi_data.values[-1].values[date_index]
                from_date = datetime.datetime.strptime(from_date, date_format)
                to_date = datetime.datetime.strptime(to_date, date_format)
                logging.debug('Received data from %s to %s for code %s', from_date, to_date, code)
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetDepositInfo(self, account_no, lookup_type=None, rqname=None, scrnno=None):
        """
        조회구분 = 1:추정조회, 2:일반조회
        """
        if rqname is None:
            rqname = '예수금상세현황요청'
        if scrnno is None:
            scrnno = '0362'
        trcode = 'opw00001'
        inputs = {
            '계좌번호': account_no,
            '비밀번호': '',
            '비밀번호입력매체구분': '00',
            '조회구분': '2' if lookup_type is None else lookup_type,
        }
        single_output = None
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if single_output is None:
                single_output = dict(zip(
                    response.listen_response.single_data.names,
                    self._RemoveLeadingZerosForNumbersInValues(response.listen_response.single_data.values, [12, 15])))
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(self._RemoveLeadingZerosForNumbersInValues(values.values, [12, 15]))
        _df = pd.DataFrame.from_records(records, columns=columns)
        return single_output

    def GetStockQuotes(self, code, rqname=None, scrnno=None):
        if rqname is None:
            rqname = '주식호가요청'
        if scrnno is None:
            scrnno = '4989'
        trcode = 'opt10004'
        inputs = {
            '종목코드': code,
        }
        single_output = None
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if single_output is None:
                single_output = dict(zip(
                    response.listen_response.single_data.names,
                    self._RemoveLeadingZerosForNumbersInValues(response.listen_response.single_data.values, [12, 15])))
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(self._RemoveLeadingZerosForNumbersInValues(values.values, [12, 15]))
        _df = pd.DataFrame.from_records(records, columns=columns)
        return single_output

    def GetOrderLogAsDataFrame1(self, account_no, order_type=None, status_type=None, code=None, rqname=None, scrnno=None):
        """
       	계좌번호 = 전문 조회할 보유계좌번호
        전체종목구분 = 0:전체, 1:종목
        매매구분 = 0:전체, 1:매도, 2:매수
        종목코드 = 전문 조회할 종목코드
        체결구분 = 0:전체, 2:체결, 1:미체결
        """

        if rqname is None:
            rqname = '실시간미체결요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opt10075'
        inputs = {
            '계좌번호': account_no,
            '전체종목구분': '0' if code is None else '1',
            '매매구분': '0' if order_type is None else order_type,
            '종목코드': '' if code is None else code,
            '체결구분': '0' if status_type is None else status_type,
        }
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(values.values)
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetOrderLogAsDataFrame2(self, account_no, order_type=None, status_type=None, code=None, order_no=None, rqname=None, scrnno=None):
        """
        종목코드 = 전문 조회할 종목코드
        조회구분 = 0:전체, 1:종목
        매도수구분 = 0:전체, 1:매도, 2:매수
        계좌번호 = 전문 조회할 보유계좌번호
        비밀번호 = 사용안함(공백)
        주문번호 = 조회할 주문번호
        체결구분 = 0:전체, 2:체결, 1:미체결
        """

        if rqname is None:
            rqname = '실시간체결요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opt10076'
        inputs = {
            '종목코드': '' if code is None else code,
            '조회구분': '0' if code is None else '1',
            '매도수구분': '0' if order_type is None else order_type,
            '계좌번호': account_no,
            '비밀번호': '',
            '주문번호': '' if order_no is None else order_no,
            '체결구분': '0' if status_type is None else status_type,
        }
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(values.values)
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetOrderLogAsDataFrame3(self, account_no, date=None, sort_type=None, asset_type=None, order_type=None, code=None, starting_order_no=None, rqname=None, scrnno=None):
        """
      	주문일자 = YYYYMMDD (20170101 연도4자리, 월 2자리, 일 2자리 형식)
        계좌번호 = 전문 조회할 보유계좌번호
        비밀번호 = 사용안함(공백)
        비밀번호입력매체구분 = 00
        조회구분 = 1:주문순, 2:역순, 3:미체결, 4:체결내역만
        주식채권구분 = 0:전체, 1:주식, 2:채권
        매도수구분 = 0:전체, 1:매도, 2:매수
        종목코드
        시작주문번호
        """

        if rqname is None:
            rqname = '계좌별주문체결내역상세요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opw00007'
        if date is None:
            now = datetime.datetime.now()
            market_start_time = now.replace(hour=8, minute=20, second=0, microsecond=0)
            if now < market_start_time:
                now = now - datetime.timedelta(days=1)
            date = now.strftime('%Y%m%d')
        inputs = {
            '주문일자': date,
            '계좌번호': account_no,
            '비밀번호': '',
            '비밀번호입력매체구분': '00',
            '조회구분': '0' if sort_type is None else sort_type,
            '주식채권구분': '0' if asset_type is None else asset_type,
            '매도수구분': '0' if order_type is None else order_type,
            '종목코드': '' if code is None else code,
            '시작주문번호': '' if starting_order_no is None else starting_order_no,
        }
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(self._RemoveLeadingZerosForNumbersInValues(values.values, 10))
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetAccaountRateOfReturnAsDataFrame(self, account_no, rqname=None, scrnno=None):
        if rqname is None:
            rqname = '계좌수익률요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opt10085'
        inputs = {
            '계좌번호': account_no,
        }
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(self._RemoveLeadingZerosForNumbersInValues(values.values, 10))
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetAccountEvaluationAsDataFrame1(self, account_no, include_delisted=True, rqname=None, scrnno=None):
        if rqname is None:
            rqname = '계좌평가현황요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opw00004'
        inputs = {
            '계좌번호': account_no,
            '비밀번호': '',
            '상장폐지조회구분': '0' if include_delisted else '1',
            '비밀번호입력매체구분': '00',
        }
        single_output = None
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if single_output is None:
                single_output = dict(zip(
                    response.listen_response.single_data.names,
                    self._RemoveLeadingZerosForNumbersInValues(response.listen_response.single_data.values, 12)))
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(self._RemoveLeadingZerosForNumbersInValues(values.values, 12))
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetAccountEvaluationAsDataFrame2(self, account_no, rqname=None, scrnno=None):
        server = self.GetLoginInfo('GetServerGubun')
        if server == '1':
            logging.warning('Not supported for simulated investment')
        if rqname is None:
            rqname = '체결잔고요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opw00005' # 모의투자에서 지원하지 않는 TR
        inputs = {
            '계좌번호': account_no,
            '비밀번호': '',
            '비밀번호입력매체구분': '00',
        }
        single_output = None
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if single_output is None:
                single_output = dict(zip(
                    response.listen_response.single_data.names,
                    self._RemoveLeadingZerosForNumbersInValues(response.listen_response.single_data.values)))
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(self._RemoveLeadingZerosForNumbersInValues(values.values))
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetAccountEvaluationAsDataFrame3(self, account_no, lookup_type=None, rqname=None, scrnno=None):
        """
        조회구분 = 1:합산, 2:개별

        [ 주의 ]
        "수익률%" 데이터는 모의투자에서는 소숫점표현, 실거래서버에서는 소숫점으로 변환 필요 합니다.
        """
        if rqname is None:
            rqname = '계좌평가잔고내역요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opw00018'
        inputs = {
            '계좌번호': account_no,
            '비밀번호': '',
            '비밀번호입력매체구분': '00',
            '조회구분': '2' if lookup_type is None else lookup_type,
        }
        single_output = None
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if single_output is None:
                single_output = dict(zip(
                    response.listen_response.single_data.names,
                    self._RemoveLeadingZerosForNumbersInValues(response.listen_response.single_data.values, [12, 15])))
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(self._RemoveLeadingZerosForNumbersInValues(values.values, [12, 15]))
        df = pd.DataFrame.from_records(records, columns=columns)
        return df

    def GetMarketPriceInfo(self, code, rqname=None, scrnno=None):
        if rqname is None:
            rqname = '시세표성정보요청'
        if scrnno is None:
            scrnno = '1010'
        trcode = 'opt10007'
        inputs = {
            '종목코드': code,
        }
        single_output = None
        columns = []
        records = []
        for response in self.TransactionCall(rqname, trcode, scrnno, inputs):
            if single_output is None:
                single_output = dict(zip(
                    response.listen_response.single_data.names,
                    response.listen_response.single_data.values))
            if not columns:
                columns = response.listen_response.multi_data.names
            for values in response.listen_response.multi_data.values:
                records.append(values.values)
        _df = pd.DataFrame.from_records(records, columns=columns)
        return single_output

    def WatchRealDataForCodesAsStream(self, codes=None, fids=None, scrnno=None, realtype=None, infer_fids=False, readable_names=False, fast_parse=False):
        if codes is None:
            codes = self.GetCommonCodeList() + self.GetKosdaqCodeList()
        if fids is None:
            fids = RealType.get_fids_by_realtype('주식시세')
        if scrnno is None:
            scrnno = '0001'
        if realtype is None:
            realtype = '0'
        for response in self.RealCall(scrnno, codes, fids, realtype, infer_fids, readable_names, fast_parse):
            yield response
